#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: watcher.pl

# This constructor / handler uses 'vzlist' to poll extended vserver status.
# Additionally it parses the vserver config files in /etc/sysconfig/vz-scripts/
# The information is then written off into CCE for GUI usage. 

use CCE;
my $cce = new CCE;

#use XML::Simple;
use Data::Dumper;

$cce->connectfd();
#$cce->connectuds(); # only for debugging

# Location of the vzlist-dump:
$vzlistdump = '/tmp/vserver-list-short';

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
}

# Generate vzlist-dump:
system("/usr/sbin/vzlist -a -o vpsid,ip,hostname,status,numproc,laverage,cpuunits,hostname | /bin/sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzlistdump");

# Pull the vzlist-dump into a Matrix:
#
# I LOVE matrix stuff! If there is a sledload of data to parse, 
# then there is hardly a more efficient way to do so.
# A legend of what colums the matrix has is at the end of the code.

# Array of VPSIDs:
@matrix_vpsids = ();

open (F, $vzlistdump) || die "Could not open $vzlistdump: $!";
$a = 0;                                     # Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
while ($line = <F>) {
    chomp($line);
    next if $line =~ /^\s*$/;               # skip blank lines
    next if $line =~ /^VEID/;               # skip lines starting with VEID
    next if $line =~ /^CTID/;               # skip lines starting with VEID
    $matrix_name = 'VPSID';
    my (@row) = split (/\s+/, $line);
    push (@{$matrix_name}, \@row);

    # Build an array that contains the VPSIDs:
    if ($VPSID[$a][0]) {                    # Plausibility check. We will not store anything into CCE if 'vpsid' is not set.
        $currentvpsid = $VPSID[$a][0];
        push (@matrix_vpsids, $VPSID[$a][0]);
    }

    # Fix for ultra-long hostnames:
    $VPSID[$a][2] = $VPSID[$a][7];

    # Check if VPS private area exists. Because it may not exist anymore, as the VPS could have been migrated off or manually have been deleted:
    $vps_home_area = "/vz/private/" . $VPSID[$a][0];
    if (-d $vps_home_area ) {
        $VPSID[$a][126] = "1";
    }
    else {
        $VPSID[$a][126] = "0";
    }
    $a++;
}
close(F);
#system("rm $vzlistdump");

# At this point we have all data from VZLIST in a matrix. But there is more data to be had in the config files. We need to get that as well:

# How many entries do we have in the Matrix?
$matrix_entries = $#VPSID;

# How many VPSIDs are in the Array?
$num_matrix_vpsids = $#matrix_vpsids;

# Set start of Matrix line:
$matrix_line = 0;

foreach $currentvpsid (@matrix_vpsids) {
    &debug_msg("Processing data for VPSID $currentvpsid\n");
    &feedthemonster;                        # Feed info into CCE
    $matrix_line++;
}

&cpu_alloc;

$cce->bye('SUCCESS');
exit(0);

#
### Subroutines:
#

# Subroutine that feeds the data into CCE:
sub feedthemonster {
    &validate;

    # Check if the VPS is already present in CCE under the old VPS ID:
    @oids = $cce->find('Vserver', {'vpsid' => $currentvpsid});

    # Check if the VPS is already present under a different VPS ID:
    @moved_oids = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][7]});

    # Legend:
    # $#oids : $#moved_oids : action
    #
    #  0 : 0  : update
    # -1 : 0  : update
    # -1 : -1 : create:

    if (($#oids == -1) && ($#moved_oids == -1)) {
        #print "Object not yet in CCE. Creating new one.\n";
        ($ok) = $cce->create('Vserver', {
            'vpsid' => $VPSID[$matrix_line][0],
            'ip' => $VPSID[$matrix_line][1],
            'hostname' => $VPSID[$matrix_line][2],
            'timestamp' => time()
            });
        $new_vps_found = '1';           # We found a new VPS, one that we have never seen before and which
                                        # is not yet in CCE. We do not want to risk to populate CCE with ALL
                                        # the found new data in one go. So let us just set the most basic info
                                        # and the timestamp. 
                                        #
                                        # We also set $new_vps_found = '1' to run feedthemonster() again.
    }
    else {
        #print "Object already present in CCE. Updating it.\n";
        # find the system object
        $new_vps_found = '0';
        &validate;

        if (($#oids < 0) && ($#moved_oids == -1)) {
                ($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
                ($ok, $sys) = $cce->get($sys_oid);
        }
        else {
                ($sys_oid) = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][2]});
                ($ok, $sys) = $cce->get($sys_oid);
        }

        ($ok) = $cce->set($sys_oid, '',{    # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsid' => $VPSID[$matrix_line][0],
                'ip' => $VPSID[$matrix_line][1],
                'hostname' => $VPSID[$matrix_line][2],
                'timestamp' => time()
            });
    }
}

# Validate the settings before we pass them on to CCE:
sub validate {

    # ip:
    if ($VPSID[$matrix_line][1] == 0) {
        $VPSID[$matrix_line][1] = "127.0.0.1";
    }
    # hostname:
    if (!$VPSID[$matrix_line][2]) {
        $VPSID[$matrix_line][2] = "localhost.localdomain";
    }

    # numproc:
    if ($VPSID[$matrix_line][4] eq "-") {
        $VPSID[$matrix_line][4] = "0";
    }
    # laverage:
    if ($VPSID[$matrix_line][5] eq "-") {
        $VPSID[$matrix_line][5] = "0";
    }
}

sub cpu_alloc {

    &debug_msg("Running cpu_alloc \n");

    ##
    # Get info about the current CPU utilization and the power of the node:
    ##

    # Location of the vzcpu-dump:
    $vzcpudump = '/tmp/vserver-cpuinfo';

    # Generate vzcpu-dump:
    system("/usr/sbin/vzcpucheck | /bin/sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzcpudump");

    if (-e $vzcpudump) {
        $b = 0;
        open(CPUDUMP,"$vzcpudump") || die("Could not find $vzcpudump. $!");
        while ($line = <CPUDUMP>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines
            if (($line =~ /^Current CPU utilization: (.*)/) && ($b == 0)) {
                $CURRENT_CPU = $1;
            }
            if (($line =~ /^Power of the node: (.*)/) && ($b == 1)) {
                $NODE_CPU = $1;
            }
            $b++;
        }
        close(CPUDUMP);
        system("/bin/rm $vzcpudump");
        &cce_update_cpu; # Write the info into CCE
    }
}

sub cce_update_cpu {

    &debug_msg("Running cce_update_cpu \n");

    ##
    # Update or populate CCE with the CPU_POWER and CURRENT_CPU (usage) of the master node:
    ##

    # This is where we actually populate CCE with the gathered info:
    unless (($CURRENT_CPU == "") || ($NODE_CPU == "")) {
        @oids = $cce->find('VserverMasterNode');
        if ($#oids < 0) {
            #print "Object not yet in CCE. Creating new one.\n";
            ($ok) = $cce->create('VserverMasterNode', {
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
            });
        }
        else {
            # find the system object
            ($sys_oid) = $cce->find('VserverMasterNode');
            ($ok, $sys) = $cce->get($sys_oid);
            ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
            });
        }
    }
}

sub debug_msg { 
    if ($DEBUG) {
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }           
}

$cce->bye('SUCCESS');
exit(0);

# 
# Copyright (c) 2016 Michael Stauber, SOLARSPEED.NET
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 
